@@ -300,6 +300,17 @@
  *         any element of A or B that makes X(j) an exact solution).
  *         If options->IterRefine = NOREFINE, berr = 1.0.
  *
+ * Glu      (input/output) GlobalLU_t *
+ *          If options->Fact == SamePattern_SameRowPerm, it is an input;
+ *              The matrix A will be factorized assuming that a 
+ *              factorization of a matrix with the same sparsity pattern
+ *              and similar numerical values was performed prior to this one.
+ *              Therefore, this factorization will reuse both row and column
+ *		scaling factors R and C, both row and column permutation
+ *		vectors perm_r and perm_c, and the L & U data structures
+ *		set up from the previous factorization.
+ *          Otherwise, it is an output.
+ *
  * mem_usage (output) mem_usage_t*
  *         Record the memory usage statistics, consisting of following fields:
  *         - for_lu (float)
@@ -338,7 +349,7 @@ sgssvx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
        SuperMatrix *L, SuperMatrix *U, void *work, int lwork,
        SuperMatrix *B, SuperMatrix *X, float *recip_pivot_growth, 
        float *rcond, float *ferr, float *berr, 
-       mem_usage_t *mem_usage, SuperLUStat_t *stat, int *info )
+       GlobalLU_t *Glu, mem_usage_t *mem_usage, SuperLUStat_t *stat, int *info )
 {
 
 
@@ -377,9 +388,9 @@ sgssvx(superlu_options_t *options, SuperMatrix *A, int *perm_c, int *perm_r,
 	rowequ = FALSE;
 	colequ = FALSE;
     } else {
-	rowequ = lsame_(equed, "R") || lsame_(equed, "B");
-	colequ = lsame_(equed, "C") || lsame_(equed, "B");
-	smlnum = slamch_("Safe minimum");
+	rowequ = strncmp(equed, "R", 1)==0 || strncmp(equed, "B", 1)==0;
+	colequ = strncmp(equed, "C", 1)==0 || strncmp(equed, "B", 1)==0;
+	smlnum = smach("Safe minimum");   /* lamch_("Safe minimum"); */
 	bignum = 1. / smlnum;
     }
 
@@ -389,17 +400,19 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 #endif
 
     /* Test the input parameters */
-    if (!nofact && options->Fact != DOFACT && options->Fact != SamePattern &&
+    if (options->Fact != DOFACT && options->Fact != SamePattern &&
 	options->Fact != SamePattern_SameRowPerm &&
-	!notran && options->Trans != TRANS && options->Trans != CONJ &&
-	!equil && options->Equil != NO)
+	options->Fact != FACTORED &&
+	options->Trans != NOTRANS && options->Trans != TRANS && 
+	options->Trans != CONJ &&
+	options->Equil != NO && options->Equil != YES)
 	*info = -1;
     else if ( A->nrow != A->ncol || A->nrow < 0 ||
 	      (A->Stype != SLU_NC && A->Stype != SLU_NR) ||
 	      A->Dtype != SLU_S || A->Mtype != SLU_GE )
 	*info = -2;
-    else if (options->Fact == FACTORED &&
-	     !(rowequ || colequ || lsame_(equed, "N")))
+    else if ( options->Fact == FACTORED &&
+	     !(rowequ || colequ || strncmp(equed, "N", 1)==0) )
 	*info = -6;
     else {
 	if (rowequ) {
@@ -428,20 +441,26 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	}
 	if (*info == 0) {
 	    if ( lwork < -1 ) *info = -12;
-	    else if ( B->ncol < 0 || Bstore->lda < SUPERLU_MAX(0, A->nrow) ||
+	    else if ( B->ncol < 0 ) *info = -13;
+	    else if ( B->ncol > 0 ) { /* no checking if B->ncol=0 */
+	         if ( Bstore->lda < SUPERLU_MAX(0, A->nrow) ||
 		      B->Stype != SLU_DN || B->Dtype != SLU_S || 
 		      B->Mtype != SLU_GE )
 		*info = -13;
-	    else if ( X->ncol < 0 || Xstore->lda < SUPERLU_MAX(0, A->nrow) ||
+            }
+	    if ( X->ncol < 0 ) *info = -14;
+            else if ( X->ncol > 0 ) { /* no checking if X->ncol=0 */
+                 if ( Xstore->lda < SUPERLU_MAX(0, A->nrow) ||
 		      (B->ncol != 0 && B->ncol != X->ncol) ||
                       X->Stype != SLU_DN ||
 		      X->Dtype != SLU_S || X->Mtype != SLU_GE )
 		*info = -14;
+            }
 	}
     }
     if (*info != 0) {
 	i = -(*info);
-	xerbla_("sgssvx", &i);
+	input_error("sgssvx", &i);
 	return;
     }
     
@@ -479,8 +498,8 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	if ( info1 == 0 ) {
 	    /* Equilibrate matrix A. */
 	    slaqgs(AA, R, C, rowcnd, colcnd, amax, equed);
-	    rowequ = lsame_(equed, "R") || lsame_(equed, "B");
-	    colequ = lsame_(equed, "C") || lsame_(equed, "B");
+	    rowequ = strncmp(equed, "R", 1)==0 || strncmp(equed, "B", 1)==0;
+	    colequ = strncmp(equed, "C", 1)==0 || strncmp(equed, "B", 1)==0;
 	}
 	utime[EQUIL] = SuperLU_timer_() - t0;
     }
@@ -513,7 +532,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	/* Compute the LU factorization of A*Pc. */
 	t0 = SuperLU_timer_();
 	sgstrf(options, &AC, relax, panel_size, etree,
-                work, lwork, perm_c, perm_r, L, U, stat, info);
+                work, lwork, perm_c, perm_r, L, U, Glu, stat, info);
 	utime[FACT] = SuperLU_timer_() - t0;
 	
 	if ( lwork == -1 ) {
@@ -522,16 +541,18 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	}
     }
 
-    if ( options->PivotGrowth ) {
-        if ( *info > 0 ) {
-	    if ( *info <= A->ncol ) {
-	        /* Compute the reciprocal pivot growth factor of the leading
-	           rank-deficient *info columns of A. */
-	        *recip_pivot_growth = sPivotGrowth(*info, AA, perm_c, L, U);
-	    }
-	    return;
+    if ( *info > 0 ) {
+        if ( *info <= A->ncol ) {
+	    /* Compute the reciprocal pivot growth factor of the leading
+	       rank-deficient (*info) columns of A. */
+	    *recip_pivot_growth = sPivotGrowth(*info, AA, perm_c, L, U);
         }
+	return;
+    }
 
+    /* *info == 0 at this point. */
+
+    if ( options->PivotGrowth ) {
         /* Compute the reciprocal pivot growth factor *recip_pivot_growth. */
         *recip_pivot_growth = sPivotGrowth(A->ncol, AA, perm_c, L, U);
     }
@@ -545,7 +566,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	    *(unsigned char *)norm = 'I';
         }
         anorm = slangs(norm, AA);
-        sgscon(norm, L, U, anorm, rcond, stat, info);
+        sgscon(norm, L, U, anorm, rcond, stat, &info1);
         utime[RCOND] = SuperLU_timer_() - t0;
     }
     
@@ -569,7 +590,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 	        Xmat[i + j*ldx] = Bmat[i + j*ldb];
     
         t0 = SuperLU_timer_();
-        sgstrs (trant, L, U, perm_c, perm_r, X, stat, info);
+        sgstrs (trant, L, U, perm_c, perm_r, X, stat, &info1);
         utime[SOLVE] = SuperLU_timer_() - t0;
     
         /* Use iterative refinement to improve the computed solution and compute
@@ -577,7 +598,7 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
         t0 = SuperLU_timer_();
         if ( options->IterRefine != NOREFINE ) {
             sgsrfs(trant, AA, L, U, perm_c, perm_r, equed, R, C, B,
-                   X, ferr, berr, stat, info);
+                   X, ferr, berr, stat, &info1);
         } else {
             for (j = 0; j < nrhs; ++j) ferr[j] = berr[j] = 1.0;
         }
@@ -599,7 +620,8 @@ printf("dgssvx: Fact=%4d, Trans=%4d, equed=%c\n",
 
     if ( options->ConditionNumber ) {
         /* Set INFO = A->ncol+1 if the matrix is singular to working precision. */
-        if ( *rcond < slamch_("E") ) *info = A->ncol + 1;
+        /*if ( *rcond < slamch_("E") ) *info = A->ncol + 1;*/
+        if ( *rcond < smach("E") ) *info = A->ncol + 1;
     }
 
     if ( nofact ) {
